# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def emulation_mka_control(self, mode, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    emulation_mka_control
		
		 Description:
		    This procedure performs control actions like start, stop or restart on MKA and does some right click options over devices.
		    The following operations are done:
		    1. Start
		    2. Stop
		    3. Restart Down
		    4. Abort
		    5. Trigger Rekey
		    6. Restart Periodic Rekey
		    7. Pause Periodic Rekey
		    8. Simulate delayed MACsec Packet Start
		    9. Simulate delayed MACsec Packet Stop
		
		 Synopsis:
		    emulation_mka_control
		        -mode            CHOICES start
		                         CHOICES restart_down
		                         CHOICES stop
		                         CHOICES abort
		                         CHOICES trigger_rekey
		                         CHOICES restart_periodic_rekey
		                         CHOICES pause_periodic_rekey
		        [-handle         ANY]
		x       [-action_indices NUMERIC
		x                        DEFAULT 0]
		
		 Arguments:
		    -mode
		        Determines the action performed on the protocol. Valid choices are:
		        restart- Restart the protocol.
		        start- Start the protocol.
		        stop- Stop the protocol.
		        restart_down- Restart the down sessions.
		        abort- Abort the protocol.
		        triggerRekey- Trigger a Rekey Event from Keysight emulated KeyServer.
		        restart Periodic Rekey - Restart the periodic rekey timer if it was stopped.
		        pause Periodic Rekey - Pause the periodic rekey timer if it is running.
		    -handle
		        This option represents the handle the user must pass to the "emulation_mka_control" procedure. This option specifies on which MKA session to control.
		x   -action_indices
		x       The list of indices over which triggerRekey, pausePeriodicRekey and RestartPeriodicRekey assync actions will occur.
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		    key:status  value:$::SUCCESS | $::FAILURE
		    If status is failure, detailed information provided.
		    key:log     value:If status is failure, detailed information provided.
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		    For mode choices restart, start, restart_down, stop and abort, MKA handle needs to be provided.
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'emulation_mka_control', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
